unit Unit1;

{$MODE objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls, Buttons, StdCtrls,
  opbitmap, opbitmapformats, lazbridge, formatdialog, ComCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnOpen: TButton;
    btnSaveAs: TButton;
    btnStream: TButton;
    cbPixelformat: TComboBox;
    Image1: TImage;
    Panel1: TPanel;
    ProgressBar1: TProgressBar;
    SaveDialog1: TSaveDialog;
    ScrollBox1: TScrollBox;
    procedure btnStreamClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnSaveAsClick(Sender: TObject);
    procedure cbPixelformatCloseUp(Sender: TObject);
  private
    { private declarations }
    fOP: TLazOPPicture;
    fOD: TOPOpenDialog;
    procedure OnSaveDetail(Sender: TObject; Bitmap: TCanvasOpBitmap; var Continue: Boolean);
    procedure OnProgress(Sender: TObject; Stage: TOPProgressStage;
      PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
      const Msg: string);
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

uses XPMime;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  fOP := TLazOPPicture.Create(Image1);
  fOP.OnProgress := @OnProgress;
  fOP.OnSaveDetails := @OnSaveDetail;
  fOD := TOPOpenDialog.Create(nil);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fOD.free;
  fOP.free;
end;

procedure TForm1.btnOpenClick(Sender: TObject);
begin
  if fOD.Execute then
  begin
    fOP.LoadFromFile(fOD.FileName);
    Image1.Transparent:=fOp.Bitmap.Transparent;
    fOP.DrawImage;
    cbPixelFormat.ItemIndex := Ord(fOP.Bitmap.PixelFormat) - 1;
    Caption := ExtractFileName(fOD.FileName) + ' (' + inttostr(fOP.Bitmap.Width) + ' x ' + inttostr(fOP.Bitmap.Height) + ')';
  end;
end;

procedure TForm1.btnSaveAsClick(Sender: TObject);
begin
  SaveDialog1.Filter := OPGSaveFilters;
  if SaveDialog1.Execute then
  begin
    fOP.SaveToFile(SaveDialog1.FileName);
  end;
end;

procedure TForm1.btnStreamClick(Sender: TObject);
var Strm: TMemoryStream;
begin
  Strm := TMemoryStream.Create;
  fOp.SaveToStream(Strm, MIME_TIF);
  fOp.LoadFromStream(Strm); //Some formats like Targa need a MimeType like fOp.LoadFromStream(Strm, MIME_TGA);
  fOp.DrawImage;
  cbPixelFormat.ItemIndex := Ord(fOP.Bitmap.PixelFormat) - 1;
  Caption := fOp.MimeType + ' (' + inttostr(fOP.Bitmap.Width) + ' x ' + inttostr(fOP.Bitmap.Height) + ')';
  Strm.free;
end;


procedure TForm1.cbPixelformatCloseUp(Sender: TObject);
begin
  fOP.Bitmap.PixelFormat := TPixelFormat(cbPixelFormat.ItemIndex + 1);
  fOP.DrawImage;
end;

//Event for getting some details about saving. Not necessary but nice ;-)

procedure TForm1.OnSaveDetail(Sender: TObject; Bitmap: TCanvasOpBitmap;
  var Continue: Boolean);
var FormatDialog: TFormatDialog;
begin
  FormatDialog := TFormatDialog.Create(nil);
  FormatDialog.Image := Bitmap;
  Continue := FormatDialog.ShowModal = mrOK;
  FormatDialog.free;
end;

procedure TForm1.OnProgress(Sender: TObject; Stage: TOPProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const R: TRect; const Msg: string);
begin
  if Stage = opsEnding then ProgressBar1.Position := 100 else
    if Stage = opsStarting then ProgressBar1.Position := 0 else
      ProgressBar1.Position := PercentDone;
  Application.ProcessMessages;
end;

initialization
{$I unit1.lrs}

end.
